 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

# Generate graphs, tables and results for each experiment
# INPUT: postproc object generated by post_proc.R (likely in cache/)
# INPUT: (files)
#   cache/str_age_ns2003_now_selected.rds
# OUTPUT: files
#   Risk-o/output/distribution_<postproc$stratum>.pdf
#   Risk-o/output/risk_<postproc$stratum>.pdf
library(dplyr)
library(ggplot2)
library(tidyr)
library(tibble)

# For testing uncomment:
# postproc <- readRDS('Risk-o/cache/postproc_V2stratum_51.rds')

with(new.env(), { #<<<<--- BEGIN ANONYMOUS ENVIRONMENT
  s <- postproc$params$parameters$stratum
  new_ns <- readRDS('Risk-o/cache/str_age_ns2003_now_selected.rds') %>% 
    filter(stratum_2003 == s) %>%
    mutate(n_2003 = n * in_2003)
  nobs <- sum(new_ns$n)
  nobs_2003 <-sum(new_ns$n_2003)
  nobs_slu <- sum(new_ns$n[new_ns$Agente_2003 == 'SLU'])
  nobs_slu_2003 <- sum(new_ns$n_2003[new_ns$Agente_2003 == 'SLU'])
  Nhats <- readRDS('Risk-o/data/previous_estimates.rds') %>% 
    filter(stratum == s) %>% 
    select(-stratum) %>%
    mutate(Nh_NAIVE = Nh_SLU_NAIVE + Nh_EST) %>%
    mutate(Nh_DIFF = Nh_SLU_DIFF + Nh_EST) 
    
  overlap_slu_labels <- sort(c(nobs_slu, nobs_slu_2003, Nhats$Nh_SLU_NAIVE, Nhats$Nh_SLU_DIFF)) %>%
    {diff(.)/diff(range(.))} %>%
    min() < 0.1
  overlap_labels <- sort(c(nobs, nobs_2003, Nhats$Nh_NAIVE, Nhats$Nh_DIFF)) %>%
    {diff(.)/diff(range(.))} %>%
    min() < 0.1
  (raw <- postproc$raw %>%
     bind_cols(., slice(Nhats, rep(1, NROW(.)))) %>%
      mutate(Method = recode(Method, Stratified = 'Direct', Aggregated = "TRC"))
  )
  (raw_SLU <- postproc$raw_SLU %>%
     bind_cols(., slice(Nhats, rep(1, NROW(.)))) %>%
      mutate(Method = recode(Method, Stratified = 'Direct', Aggregated = "TRC"))
   
  ) 
  (prediction_risk <- postproc$prediction_risk%>%
      ungroup %>%
     bind_cols(., slice(Nhats, rep(1, NROW(.)))) %>%
      mutate(Method = recode(Method, Stratified = 'Direct', Aggregated = "TRC"))
  )
  (prediction_risk_SLU <- postproc$prediction_risk_SLU %>%
     bind_cols(., slice(Nhats, rep(1, NROW(.)))) %>%
      ungroup %>%
      mutate(Method = recode(Method, Stratified = 'Direct', Aggregated = "TRC"))
  ) 

    
d.nobs <- (nobs - min(raw$N))/(max(raw$N) - min(raw$N)) * nlevels(raw$experiment)
risk_SLU <- prediction_risk_SLU %>%
  #  mutate(N = round((N - nobs_slu) / nobs_slu * 100, digits = 0))  %>%
  ggplot() +
  theme_bw() +
  ggtitle('Estimation Risk by Population Size (SLU)', subtitle = postproc$stratum) +
  aes(x =  N, lty = Method) +
  geom_rect(aes(xmin = -Inf, xmax = nobs_slu, ymin  = -Inf, ymax = Inf), fill = 'gray90') +
  geom_line(aes(y = MSE)) +
  #geom_vline(xintercept = n) +
  geom_vline(xintercept = nobs_slu, lwd  = 1) +
  geom_vline(xintercept = Nhats$Nh_SLU_NAIVE, lty = 2) +
  geom_vline(xintercept = Nhats$Nh_SLU_DIFF, lty = 2) +
  geom_vline(xintercept = nobs_slu_2003, lty = 2) +
  scale_x_continuous(
    breaks = waiver(), 
    labels = function(x){
      l <- round((x-nobs_slu)/nobs_slu * 100,digits = 0)
      sl <- ifelse(l>=0, ' +', ' -')
      paste0(x, '\n(',sl,  abs(l), '%)')
    },
      sec.axis = sec_axis(trans = ~., breaks = c(nobs_slu_2003, Nhats$Nh_SLU_NAIVE, Nhats$Nh_SLU_DIFF, nobs_slu), 
                     labels = c('nobs (2003)', 'Nh (Direct)', 'Nh (TRC)', 'nobs (2018)')), 
                     position = 'bottom'  
  ) +
  geom_text(aes(x = nobs_slu - 10, y = max(MSE), label = 'Imposibility\n Region', vjust = 'top', hjust = 'right'), check_overlap = TRUE) +
  ylim(0,NA) +
  xlab('True N') +
  ylab('MSE')
if(overlap_slu_labels){
  risk_SLU <- risk_SLU + theme(axis.text.x.top = element_text(angle=30, hjust = 0))
}
plot(risk_SLU)


pdf(10, 5, 
  file = postproc$params$filename(
    naming_field = 'COMP_ID', rule = 'Risk-o/output/risk_SLU_%.pdf'
  )
)
plot(risk_SLU);dev.off()
}) #<<<<--- END ANONYMOUS ENVIRONMENT
