 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

# Process ONE simulation into usable datasets.
# INPUT: list "res" with results from simulations generated by underpowered.R (most likely loaded from Risk-o/cache)
# OUTPUT: list "processed" with post-processed simulation information. 

library(dplyr)
library(ggplot2)
library(tidyr)
library(tibble)

# For debugging uncomment next line:
#res <- readRDS('Risk-o/cache/stratum_11.rds')

with(new.env(),{
#with(environment(),{
  
  p <- res[[1]]$params
  stratum <- paste('Based on data from stratum', p$parameters$stratum)
  Nhat_SLU <- sapply(res, FUN = function(x)x$stratif_SLU)
  Nhat_EST <- sapply(res, FUN = function(x)x$stratif_EST)
  Nhat_comb <- sapply(res, FUN = function(x)x$aggreg)
  Nhat_SLU_comb <- Nhat_comb - Nhat_EST
  N_EST <- res[[1]]$params$parameters$N_EST
  N_SLU <- res[[1]]$params$parameters$N_SLU
  N_COMB <- N_EST + N_SLU
  Nhat_estrat <- Nhat_SLU + Nhat_EST
  
  estim <- bind_cols(
    N = N_COMB,
    Nhat_estrat %>% 
      apply(., MARGIN = 2, FUN = quantile, c(0.025, 0.5, 0.975),na.rm = T, names = FALSE) %>%
      t %>% as.tibble %>%
      transmute(strat.LI = V1, strat.M = V2, strat.LS = V3),
    Nhat_comb %>% 
      apply(., MARGIN = 2, FUN = quantile, c(0.025, 0.5, 0.975),na.rm = T, names = FALSE) %>%
      t %>% as.tibble %>%
      transmute(comb.LI = V1, comb.M = V2, comb.LS = V3)
  )
  
  estim_SLU <- bind_cols(
    N = N_SLU,
    Nhat_SLU %>% 
      apply(., MARGIN = 2, FUN = quantile, c(0.025, 0.5, 0.975),na.rm = T, names = FALSE) %>%
      t %>% as.tibble %>%
      transmute(strat.LI = V1, strat.M = V2, strat.LS = V3),
    Nhat_SLU_comb %>% 
      apply(., MARGIN = 2, FUN = quantile, c(0.025, 0.5, 0.975),na.rm = T, names = FALSE) %>%
      t %>% as.tibble %>%
      transmute(comb.LI = V1, comb.M = V2, comb.LS = V3)
  )
  
  (raw <- bind_rows( 
    Nhat_estrat %>%
      as.tibble() %>%
      rename_all(funs(paste0('Stratified_', .))) %>%
      gather(key = 'Experiment', value = 'Nhat') %>%
      separate(Experiment, c('Method', 'experiment'), sep = '_'),
    Nhat_comb%>%
      as.tibble() %>%
      rename_all(funs(paste0('Aggregated_', .))) %>%
      gather(key = 'Experiment', value = 'Nhat') %>%
      separate(Experiment, c('Method', 'experiment'), sep = '_')
  ) %>%
      mutate(experiment = as.numeric(gsub(pattern = 'V', x = experiment, replacement = ''))) %>%
      mutate(N = N_COMB[experiment], experiment = factor(N))
  )
  (raw_SLU <- bind_rows( 
    Nhat_SLU %>%
      as.tibble() %>%
      rename_all(funs(paste0('Stratified_', .))) %>%
      gather(key = 'Experiment', value = 'Nhat') %>%
      separate(Experiment, c('Method', 'experiment'), sep = '_'),
    Nhat_SLU_comb%>%
      as.tibble() %>%
      rename_all(funs(paste0('Aggregated_', .))) %>%
      gather(key = 'Experiment', value = 'Nhat') %>%
      separate(Experiment, c('Method', 'experiment'), sep = '_')
  ) %>%
      mutate(experiment = as.numeric(gsub(pattern = 'V', x = experiment, replacement = ''))) %>%
      mutate(N = N_SLU[experiment], experiment = factor(N))
  )
  prediction_risk <- raw %>% 
    mutate(sqdiff = ((N - Nhat)^2)) %>%
    mutate(absdiff = abs(N - Nhat)) %>%
    group_by(Method, N) %>%
    summarize(
      MSE = mean(sqdiff, na.rm = TRUE),
      MAE = mean(absdiff, na.rm = TRUE)
    )
  
  prediction_risk_SLU <- raw_SLU %>% 
    mutate(sqdiff = ((N - Nhat)^2)) %>%
    mutate(absdiff = abs(N - Nhat)) %>%
    group_by(Method, N) %>%
    summarize(
      MSE = mean(sqdiff, na.rm = TRUE),
      MAE = mean(absdiff, na.rm = TRUE)
    )
  
  prob_estimable <-  raw %>%
    group_by(Method, N) %>%
    summarize(P_estimable = mean(!is.na(Nhat)))
  
  prob_estimable_SLU <-  raw_SLU %>%
    group_by(Method, N) %>%
    summarize(P_estimable = mean(!is.na(Nhat)))
  
  assign('processed', 
         list(
           params = p,
           stratum = stratum,
           raw = raw, 
           raw_SLU = raw_SLU, 
           estim = estim, 
           estim_SLU = estim_SLU, 
           prediction_risk = prediction_risk, 
           prediction_risk_SLU = prediction_risk_SLU, 
           prob_estimable = prob_estimable,
           prob_estimable_SLU = prob_estimable_SLU
         ), 
         envir = parent.env(environment())
  )
})

