 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

# Simulates capture-recapture with data from one stratum of CVR 2003.
#INPUT: 
# -p (in memory): object of type cr_simu_params with parameters for the simulation
#
#OUTPUT: (in memory) list 'results' in calling environment
#
#Working Dir: Project root.
#
#Depends on files:
#   'src/parameters.R'
#   'Risk-o/src/fn_Risk-o.R'
#   'data/Data_2003'

library(Rcapture) 
library(dplyr)
library(tidyr)
library(tibble)
source('src/parameters.R')
source('Risk-o/src/fn_Risk-o.R')

##For debugging execute
#  p <- cr_simu_params$new()
#  p$load_values_from_list(jsonlite::read_json('Risk-o/cache/V2stratum_25.json', simplifyVector = T))

with(new.env(),{   #<<--- START ANONYMOUS ENVIRONMENT
  data_2003 <- readRDS('data/Data_2003.rds')
  
  #create all the needed smoothed tables
  versions <- list()
  for (i in seq_along(p$parameters$N_EST)){
    x <- full_table(
      data_2003 %>% filter(estrato == p$parameters$stratum), 
      est_SLU = p$parameters$N_SLU[i], 
      est_EST = p$parameters$N_EST[i]
    )
    smooth <- fn_get_probs(model = p$parameters$smoothing_model, data = x)
    Nest <- x %>% filter(Agente == 'EST') %>% summarize(sum(n)) %>% as.numeric
    Nslu <- x %>% filter(Agente == 'SLU') %>% summarize(sum(n)) %>% as.numeric
    N <- x %>% summarize(sum(n)) %>% as.numeric
    versions[[i]] <- list(smooth = smooth, Nest = Nest, Nslu = Nslu, N = N)
  }
  
  #Simulation: 
  library(snowfall)
  snowfall::sfInit(parallel = TRUE, cpus = 4, type = 'SOCK')
  snowfall::sfLibrary(dplyr)
  snowfall::sfLibrary(tibble)
  
  for(i in seq_along(versions)){
    stratif <- selected <- numeric(p$parameters$trials)
    res <- snowfall::sfSapply(
      x = 1:p$parameters$trials,
      fun = function(y, smooth){
        source('Risk-o/src/fn_Risk-o.R', local = TRUE)
        syn <- fn_synth(smooth)
        stratif_SLU = cr_direct(syn$SLU)$Nhat
        stratif_EST = cr_direct(syn$EST)$Nhat
        aggreg = cr_direct(syn$COMB)$Nhat
        return(c(stratif_SLU, stratif_EST, aggreg))
      },
      smooth = versions[[i]]$smooth
    )
    versions[[i]]$stratif_SLU<-  as.numeric(res[1,])
    versions[[i]]$stratif_EST<- as.numeric(res[2,])
    versions[[i]]$aggreg <- as.numeric(res[3,])
    versions[[i]]$params <- p
    versions[[i]]$date <- date()
    assign('results', versions, envir = parent.env(environment()))
  }
}) # <<--- END ANONYMOUS ENVIRNMENT
