 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

library(R6)
library(yaml)
library(jsonlite)

param_utils <- new.env()
with(param_utils,{ # <<<--- ENCLOSE IN ENVIRONMENT FOR NAME  MANAGEMENT
  CParams <- R6::R6Class(
    'CParams',
    active = list(
      parameters = function(value){
        if (missing(value)){
          return(private$params)
        } else {
          stop("'parameters' is read-only. Use (function) instead.")
        }
      },
      date = function(){
        return(private$date_data)
      }
    ),
    private = list(
      params = NULL,
      date_data = NULL,
      add = function(...){
        list_params <- list(...)
        if(!is.list(list_params)) stop('Input must be a named list of parameters')
        if(sum(unique(names(list_params)) != '') != length(list_params)){
          stop('Every parameter must have a different name')
        } 
        if( length(intersect(names(self$params), names(list_params))) > 0)
          stop('Cannot have repeated parameter names')
        private$params <- append(private$params, list_params)
      },
      LOADED = FALSE
    ),
    public = list(
      load_values = NULL,
      initialize = function(...){
        #create fields and a function to load those fields
        a <- match.call(expand.dots = FALSE)$`...`
        #A more robust way of creating the function (try it later)
        #a <- function(){}
        #formals(a) <- eval(parse(text = 'alist(x = 1, b = NULL)'))
        #body(a) <-  as.call(c(as.name('{'), parse(text = 'x + b; print(\'hola\')')))
        
        bod <- paste(paste0('private$params$',paste(names(a), names(a), sep = ' = ')), collapse = '\n')
        bod <- paste0('if(private$LOADED){warning(\'already loaded\')} else {private$LOADED <- TRUE}\n',bod)
        private$params <- list()
        private$add(...)
        self$load_values <- eval(parse( 
           text =  paste0('function(){', bod, '}')
        ))
        formals(self$load_values) <- a
        private$date_data <- date()
        invisible(self)
      },
      yaml = function(file = NULL){
        yaml::as.yaml(private$params)
      },
      json = function(file = NULL){
        jsonlite::toJSON(private$params, pretty = TRUE)
      },
      equal = function(obj){
        #Override this method for altering the equality criterion (e.g. make it invariant to permutations)
        identical(self$parameters, obj$parameters)
      },
      print = function(){
        cat( paste(names(self$parameters), self$parameters, sep = ' = '),  sep = '\n')
      },
      load_values_from_list = function(lista){
        do.call(self$load_values, lista)
      },
      filename = function(naming_field, rule = '%.RData'){
        gsub(pattern = '%', replacement = private$params[[naming_field]], x = rule)
      },
      load_cache = function(cached_params_name = 'params',
                             naming_field ='COMP_ID',   file_pattern = 'LARGE_STORAGE/%.rds'){
        #check if a computation file already exists, and read it if does
        fn <- self$filename(naming_field = naming_field, rule = file_pattern)
        if(file.exists(fn)) {
          x <- readRDS(fn)
          if(self$equal(x[[cached_params_name]])){
            return(x)
          }
        }
        return(NULL)
      }
    )
  )
  
  `==.CParams` <<- function(a, b){
    a$equal(b)
  }
  
  `!=.CParams` <<- function(a, b){
    !a$equal(b)
  }
  
  
  fn_batch_parameters <- function(yaml.file, param_class, sep = ''){
    library(yaml)
    library(R6)
    yread <- yaml::yaml.load_file(input = yaml.file)
    if(is.null(yread$default)){
      stop('No default parameters')
    }
    default <- yread$default
    yread$default <- NULL
    res <- list() 
    for (i in seq_along(yread)){
      nam <- names(yread)[i]
      curr <- default
      curr[names(yread[[i]])] <- yread[[i]]
      if('seq' %in% names(yread[[i]])){
        suf <- yread[[i]]$seq
        curr$seq <- NULL
        for (j in suf){
          #  create new objects for each element of the sequence
          name_ele <- paste(nam, j, sep = sep)
          res[[ name_ele ]] <- param_class$new()
          curr2 <- curr
          newn <- yread[[i]]
          newn$seq <- NULL
          curr2[ names(newn) ] <- paste(newn, j, sep = sep)
          res[[ name_ele ]]$load_values_from_list(curr2)
        }
      } else {
        res[[nam]] <- param_class$new()
        res[[nam]]$load_values_from_list(curr)
      }
    }
    return(res)
  }
  
  fn_batch_filenames <- function(batch, naming_field, rule = '%.RData'){
    if (prod(sapply(batch, FUN = function(x) naming_field %in% names(x$parameters), simplify = TRUE)) == 0){
      stop('naming_field not present in all members')
    }
    res <- list()
    for (x in seq_along(batch)){
      res[[ names(batch)[x] ]] <- batch[[x]]$filename(naming_field = naming_field, rule = rule)
    }
    res
  }
  
}) ## <-- END ENCLOSING ENVIRONMENT

#Examples:
#Example of use
#CBasic_param <- R6::R6Class(
#  'basic_params', inherit = param_utils$CParams,
#  public = list(
#    initialize = function(){
#      super$initialize(
#        COMP_ID = 'TEST_COMP',
#        agente_col = 'Agente',
#        bi = 10,
#        geo_col = 'stratum_david4',
#        lcmcr_lists = c('in_CICR', 'in_CEDAP', 'in_DP', 'in_COMISEDH', 'in_CNDDHH', 'in_CVR', 'in_MIMDES'),
#        loglin_lists = c('in_CICR', 'in_CEDAP', 'in_DP', 'in_COMISEDH', 'in_CNDDHH', 'in_CVR', 'in_MIMDES'),
#        nsam = 500,
#        th = 50,
#        imp_source = 'src/imputation_3.R',
#        max_order_ll = 2,
#        min_list_size = 4
#      )
#    }
#    # add code to override the equal method for other ways of comparing
#    #equal = function(q){
#    #  super$equal(q)    
#    #}
#  )
#)
#a <- param_utils$fn_batch_parameters(yaml.file = 'batch_def/coarse.yml', param_class = CBasic_param)
#param_utils$fn_batch_filenames(a, naming_field = 'COMP_ID', rule = 'LARGE_STORAGE/4_stratified/basic_run_raw_%.RData') %>% 
#  unlist %>%
#  paste(., collapse = '\n') %>%
#  cat
#a[[1]]$filename(naming_field = 'COMP_ID')
#b <- CBasic_param$new()
